@@ -119,7 +119,6 @@ def list_keys(
                 typer.echo(f"{contar_http}{obj.key}")
             elif key_methods == "key":
                 typer.echo(f"{obj.key}")
-                return obj.key
             elif key_methods == "size":
                 # TODO add a kwarg to get total size
                 typer.echo(f"{obj.key} -> {round(obj.size / 1024 ** 2, 2)}Mb")
@@ -275,7 +274,7 @@ def change_permissions(
         typer.echo(e)
 
 
-def _deleter(k: str, prompt):
+def _deleter(k: str, prompt, feedback: bool = True):
     contents, s3, bucket_name, _ = get_login()
 
     if prompt:
@@ -292,9 +291,11 @@ def _deleter(k: str, prompt):
 
     else:
         s3.Object(bucket_name, k).delete()
-        message = "Deleted Key: "
-        deleted = typer.style(f"{k}", fg=typer.colors.RED)
-        typer.echo(message + deleted)
+        if feedback:
+            message = "Deleted Key: "
+            deleted = typer.style(f"{k}", fg=typer.colors.RED)
+            typer.echo(message + deleted)
+        return
 
 
 @app.command()
@@ -529,5 +530,98 @@ def create_upload_list(
     return
 
 
+@app.command("move-object")
+def move_object(
+    destination_path: str = typer.Argument(
+        ..., help="Destination path. Don't include the delimiter!"
+    ),
+    origin_files: List[str] = typer.Option(
+        None, "--files", "-f", help="Choose one or more objects you wish to move."
+    ),
+    permission: ACLTypes = typer.Option(
+        ACLTypes.public_read.value,
+        "--permissions",
+        "-p",
+        help="Sets the permission for the copied object.",
+    ),
+    threads: int = typer.Option(
+        3, "--threads", "-t", help="Amount of threads used to upload in parallel."
+    ),
+):
+    """
+    Moves objects from one location to another within the same bucket.
+    """
+    _, s3, bucket_name, client = get_login()
+
+    if destination_path[-1] == "/":
+        typer.echo("Destination path should not end with '/'")
+        raise typer.Exit()
+
+    file_dict = {}
+
+    for f in origin_files:
+        splitted_file_name = f.split("/")
+
+        file_dict[f] = f"{destination_path}/{splitted_file_name[-1]}"
+
+    try:
+        executor = ThreadPoolExecutor(max_workers=threads)
+        futures = [
+            executor.submit(_move_obj, permission, s3, client, bucket_name, orig, dest)
+            for orig, dest in file_dict.items()
+        ]
+        for f in futures:
+            f.result()
+
+    except Exception as e:
+        if (
+            str(e)
+            == "An error occurred (404) when calling the CopyObject operation: Not Found"
+        ):
+            typer.echo("Origin object not found!")
+            raise typer.Exit()
+        else:
+            typer.echo(
+                "An error occurred! Please check if the origin object path is correct"
+            )
+            raise typer.Exit()
+
+    try:
+        # Delete origin_file
+        executor = ThreadPoolExecutor(max_workers=threads)
+        futures = [
+            executor.submit(_deleter, k, False, feedback=False) for k in origin_files
+        ]
+        for f in futures:
+            f.result()
+
+    except Exception as e:
+        raise typer.Exit()
+
+
+def _move_obj(permission, s3, client, bucket_name, orig, dest):
+    obj_name = orig.split("/")[-1]
+
+    response = client.get_object(Bucket=bucket_name, Key=orig)
+    obj_size = response.get("ContentLength")
+    content_type = response.get("ContentType")
+
+    progbar = tqdm(
+        total=obj_size, unit="B", unit_scale=True, unit_divisor=1024, desc=obj_name
+    )
+
+    def upload_progress(chunk):
+        progbar.update(chunk)
+
+    copy_source = {"Bucket": bucket_name, "Key": orig}
+    bucket = s3.Bucket(bucket_name)
+    bucket.copy(
+        copy_source,
+        dest,
+        Callback=upload_progress,
+        ExtraArgs={"ContentType": content_type, "ACL": permission.value},
+    )
+
+
 if __name__ == "__main__":
     app()
